/***************************************************************************
 *   Copyright (C) 2007 by Jan Dzik, Alexios Kotsifakos, Harambos Nikolaou,*
 *   Kastrisios Konstantinos, Giorgos Pletsis, Kandias Miltiadis           *
 *   iam.jss@gmail.com   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/




/****************************
    LIBRARY DECLARATIONS
 ****************************/

//#ifdef HAVE_CONFIG_H
//#include <config.h>
//#endif
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <pthread.h>
#include "scannerRQ/scannerRQ.h"
#include "scannerRQ/errorsRQ.h"


extern int RQ_Error;

/****************************
    CONSTANT DEFINITIONS
 ****************************/

#define MAX_SERVER_NAME         50
#define MAX_THREADS             50
#define MAX_ERROR_MSG           512
#define printError(s, e) fprintf(stderr, "%s: %s\n", s, strerror(e))
#define DEFAULT_THREADS         5



/****************************
    TYPE DEFINITIONS
 ****************************/

typedef struct vulnDesc* vulnDescPtr;

typedef struct vulnDesc 
{
	vulnDescPtr  nextDesc;
	char*        DescMsg;
}vulnDesc;


/*********************************
    AUXILIARY FUNCTION PROTOTYPE
 *********************************/

void* sendRequests (void* arg);


vulnDescPtr* threadScan;



int numarg(char *p) {
	int i;
	for (i = 0 ; p[i] ; i++)
		if (!isdigit(p[i] && !isspace(p[i]) && p[i] != '+' && p[i] != '-')
			return 0;
	return 1;
}

/****************************
       SCANNER FUNCTION 
 ****************************/

int main(int argc, char *argv[]) 
{
	// local variables
	char *serverName;
	int i, err, par = 0, num = 0, totalThreads = DEFAULT_THREADS;
	pthread_t* tids;

	// check for invalid arguments
	if(argc < 2 || argc > 4)
	{
		fprintf(stderr, "\nInvalid number of arguments.\n");
		return (-1);
	}
	
	// get server's name
//	if(strlen(argv[1]) >= MAX_SERVER_NAME)
//	{
//		printf("\nServer name must be up to %d bytes!\n",MAX_SERVER_NAME);
//		return (-1);
//	}
	serverName = strdup(argv[1]);
//    	strcpy ( serverName, argv[1]);

	// get total number of threads 
	for (i = 2 ; i < argc ; ++i) {
		if (numarg(argv[i]) && num++)
			totalThreads = atoi(argv[i]);
		else if (!strcmp(argv[i], "-p") && par)
			par++;
		else {
			fprintf(stderr, "\nInvalid arguments.\n");
			return (-1);
		}
	}
	if (totalThreads < 0) {
		fprintf(stderr, "\nInvalid thread number. Working with default number of threads.\n");
		totalThreads = DEFAULT_THREADS;
	}
	totalThreads = atoi(argv[2]);
	if( !(totalThreads > 0  &&  totalThreads <= MAX_THREADS) )
	{
		fprintf(stderr, "\nNumber of threads should be between 1 and 50!\n");
		return (-1);
	}

	
	// construct and initialize array to contain vulnerability descriptions
	if( (threadScan = (vulnDescPtr*) malloc (totalThreads * sizeof(vulnDescPtr))) == NULL)
	{
		perror("malloc");
		return (-1);
	}
	for(i=0;i<totalThreads;++i)
	{
		if( (threadScan[i] = (vulnDescPtr) malloc (totalThreads * sizeof(vulnDescPtr))) == NULL)
		{
			perror("malloc");
			return (-1);
		}
		threadScan[i]->nextDesc = NULL;
		threadScan[i]->DescMsg = NULL;
	}
		

	//initiallize RQ-level
	if ((err = RQ_Init(totalThreads, serverName, 1)) < 0) {
		RQ_PrintError(err);
		exit(-1);
	}

    	// allocate space for a table containing all thread-IDs 
    	if( (tids = (pthread_t*) malloc(totalThreads * sizeof(pthread_t))) == NULL)
	{
		perror("malloc");
		return (-1);
	}

	// create the requested number of threads that will perform the scan
	for(i=0;i<totalThreads;++i)
		if( err = pthread_create(tids+i, NULL, sendRequests, (void*)i))
		{
			printError("pthread_create", err);
			return (-1);
		}

	// wait for all threads to finish scanning the requested url
	for(i=0;i<totalThreads;++i)
		if( err = pthread_join(*(tids+i), NULL) )
		{
			printError("pthread_join", err);
			return (-1);
		}

	// clean RQ-level
	if ((err = RQ_Cleanup()) < 0) {
		RQ_PrintError(err);
	}

	// print scan's results
	printf("\nThe following vulnerabilities were diagnosed...");
	
	for(i=0; i<totalThreads; ++i)
	{
		printf("\nthread %d\n-----------------------------------------------", i);
		while( threadScan[i]->DescMsg != NULL )
		{	
			printf("\n%s", threadScan[i]->DescMsg);

			if(threadScan[i]->nextDesc == NULL)
				threadScan[i]->DescMsg = NULL;
			else
				threadScan[i] = threadScan[i]->nextDesc;
		}
		printf("\n-----------------------------------------------\n");
	}


	return EXIT_SUCCESS ;
}






/****************************
     AUXILIARY FUNCTION 
 ****************************/

void* sendRequests (void* arg)
{
	// local variables
	requestInfo_t  rinf;
	
	vulnDescPtr    currDescript = NULL;
	vulnDescPtr    prevDescript = NULL; 
	//vulnDescPtr    head = (vulnDescPtr)arg;
	int tid = (int) arg;
	int i, flag = 0, err;
	char **urls;


    // keep sending requests to server until all requests have been served

//    crawler_run(serverName, urls);
//    for (i = 1 ;;) {	
        while (RQ_SendNextRequest(tid, &rinf) != RQ_SCANNER_EOR) 
        {
	    // if sth goes wrong print an appropriate message and go on with the next request
	    if (RQ_Error != RQ_SCANNER_OK) 
	    {
		    RQ_PrintError(RQ_Error);
		    continue;
	    }

	    if( strstr( rinf.httpResponse, rinf.httpExpectedResponse) != NULL )
	    {		
			if(currDescript != NULL)
				prevDescript = currDescript;

			currDescript = (vulnDescPtr) malloc( sizeof(vulnDescPtr) );

			currDescript->DescMsg = rinf.description;
			currDescript->nextDesc =NULL;

			if(prevDescript != NULL)
				prevDescript->nextDesc = currDescript;	
					
			if(flag == 0  &&  prevDescript != NULL)
			{
				threadScan[tid]->DescMsg = prevDescript->DescMsg;
				threadScan[tid]->nextDesc = prevDescript->nextDesc;
				flag = 1;
			}			
	    }
	    free(rinf.httpResponse); rinf.httpResponse=NULL;
        }
//        if (par || urls[++i] != NULL) break;
//        free(serverName);
//        serverName = strdup(urls[i]);
    }
 
    pthread_exit(NULL);
}
