/***************************************************************************
 *   Copyright (C) 2007 by Jan Dzik, Alexios Kotsifakos, Harambos Nikolaou,*
 *   Kastrisios Konstantinos, Giorgos Pletsis, Kandias Miltiadis           *
 *   iam.jss@gmail.com   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* system includes */
#include <string.h>

/* project includes */
#include "scannerdb.h"
#include "errors.h"

typedef struct _buffer {
	/* reference count */
	/*int _refs;*/
	/* sqlite3 connection */
	sqlite3 *_sqlitedb;
	/* table buffer */
	char **_table;
	/* number of rows */
	int _rows;
	/* number of columns */
	int _columns;
} DBbuffer_t;

static DBbuffer_t DBbuffer;

int DB_init(void) {
	int result=0;
	char *sqlError=NULL;
	
	memset(&DBbuffer, 0, sizeof(DBbuffer_t));
	
	result=sqlite3_open(SCANNERDB_DB_FILE, &DBbuffer._sqlitedb);
	if(result!=SQLITE_OK) {
		return SCANNER_DB_OPEN;
	}
	
	result=sqlite3_get_table(
		DBbuffer._sqlitedb,
		"SELECT * FROM Requests WHERE id<1000",
		&DBbuffer._table,
		&DBbuffer._rows,
		&DBbuffer._columns,
		&sqlError);
	
	if(result!=SQLITE_OK) {
		sqlite3_free(sqlError);
		return SCANNER_DB_QUERY;
	}
	
	return SCANNER_OK;
}

int DB_fini(void) {
	int result=0;
	
	sqlite3_free_table(DBbuffer._table);
	
	result=sqlite3_close(DBbuffer._sqlitedb);
	if(result!=SQLITE_OK) {
		return SCANNER_DB_CLOSE;
	}
	
	memset(&DBbuffer, 0, sizeof(DBbuffer_t));
	
	return SCANNER_OK;
}

int DB_open(scannerdb_t **db) {
	/* allocate handler */
	*db=(scannerdb_t *)malloc(sizeof(scannerdb_t));
	if(*db==NULL) {
		return SCANNER_NOMEM;
	}
	memset(*db, 0, sizeof(scannerdb_t));
	
	return SCANNER_OK;
}

int DB_close(scannerdb_t *db) {
	int result=0;
	
	free(db);
	
	return SCANNER_OK;
}

int DB_get_lines_num(scannerdb_t *db) {
	return DBbuffer._rows;
}

int DB_get_columns_num(scannerdb_t *db) {
	return DBbuffer._columns;
}

int DB_line_next(scannerdb_t *db) {
	++db->_current_row;
	
	if(db->_current_row>DBbuffer._rows)
		db->_current_row=0;
	
	return db->_current_row;
}

int DB_get_current_line(scannerdb_t *db) {
	return db->_current_row;
}

int DB_go_to_line(scannerdb_t *db, int line) {
	if(line<0 || line>=DBbuffer._rows) {
		return SCANNER_BADPARAM;
	}
	
	db->_current_row=line;
	
	return db->_current_row;
}

char *DB_get_field(scannerdb_t *db, int column) {
	if(column<0 || column>=DBbuffer._columns)
		return NULL;
	
	return DBbuffer._table[DBbuffer._columns*(db->_current_row+1)+column];
}
