/***************************************************************************
 *   Copyright (C) 2007 by Jan Dzik, Alexios Kotsifakos, Harambos Nikolaou,*
 *   Kastrisios Konstantinos, Giorgos Pletsis, Kandias Miltiadis           *
 *   iam.jss@gmail.com   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SCANNERDB_H
#define SCANNERDB_H

/* system includes */
#include <stdlib.h>
#include <sqlite3.h>

/* The scanner database has only one table, the requests it will perform */
/* This api is serves as an abstraction of the sqlite database used to store that table */
/* The sqlite version used is v.3 */
/* :D */

/* the name of the scanner sqlite database file */
#define SCANNERDB_DB_FILE "scanner.db"

/* Scanner database handler */
/* Obtain a handler by calling DB_open() */
typedef struct _scannerdb {
	/* current row */
	int _current_row;
} scannerdb_t;

/* initialize DB */
/* Call this function before any other DB_*() function calls */
/* returns; SCANNER_OK on success, <0 on failure */
int DB_init(void);

/* finalize DB */
/* Call this function when all work with the db has finished */
/* returns; SCANNER_OK on success, <0 on failure */
int DB_fini(void);

/* Open the scanner database table
returns; SCANNER_OK on success
Each time DB_Open() is called a new copy of the requests table is
brought to memory. If multiple threads are to use the DB call this
function in the main thread before any other threads are created.
When all threads have finished, call DB_close() to free the handler
resources and memory used.
<0 on errror
*/
int DB_open(scannerdb_t **db);

/* Close the scanner database table and free resources */
/* returns; 0 if sucessfull, non-zero otherwise */
int DB_close(scannerdb_t *db);

/* Get number of lines in the database table */
/* returns; the number of records in the table */
/* <0 if error occured */
int DB_get_lines_num(scannerdb_t *db);

/* Get number of columns in the table */
/* returns; the number of columns in the table */
/* <0 if error occured */
int DB_get_columns_num(scannerdb_t *db);

/* Move the handler line pointer to the next line */
/* returns; the current line number. */
/* <0 in case of error */
/* Line counting starts with 0 */
/* Resets to first line when called at the last */
int DB_line_next(scannerdb_t *db);

/* Get the line number the handler is pointing to */
/* Note: line indexing starts at 0. */
/* returns; current line number or <0 in case of error */
int DB_get_current_line(scannerdb_t *db);

/* Move the handler's line pointer to a specified line */
/* returns; the line number the handler points to after
the function call or <0 in case of error */
int DB_go_to_line(scannerdb_t *db, int line);

/* Get pointer to a field in the current line */
/* returns NULL if out of bounds parameter is used */
/* column indexing starts at 0 */
char *DB_get_field(scannerdb_t *db, int column);

#endif
