/***************************************************************************
 *   Copyright (C) 2007 by Jan Dzik, Alexios Kotsifakos, Harambos Nikolaou,*
 *   Kastrisios Konstantinos, Giorgos Pletsis, Kandias Miltiadis           *
 *   iam.jss@gmail.com   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* system includes */
#include <string.h>
#include <pthread.h>

/* project includes */
#include "scannerRQ.h"
#include "errorsRQ.h"

int RQ_Error = RQ_SCANNER_OK;

int RQ_Init(int nthreads, char *domain, int md5sum)
{
	char *dom = domain;
	if (DB_init() < 0) {
		RQ_Error = RQ_SCANNER_DB_ERROR;
		return RQ_Error;
	}
	
	md5Calculation = md5sum;

	/* strip the last character if it is the '/' */
	if (*(dom + strlen(dom) - 1) == '/') {
		*(dom + strlen(dom) - 1) = '\0';
	}	
	memcpy(url, dom, URL_LENGTH);
	threadInfoSize = nthreads;
	threadInfo = (scannerRQ_t *) malloc(nthreads * sizeof(scannerRQ_t));
	
	if (threadInfo == NULL) {
		RQ_Error = RQ_SCANNER_NOMEM;
		return RQ_Error;
	}

	
	if (RQ_Open() < 0) {
		RQ_Error = RQ_SCANNER_OPEN;
		return RQ_Error;
	}

	RQ_Error = RQ_SCANNER_OK;
	return RQ_SCANNER_OK;
}

int RQ_Cleanup(void)
{
	int i;

	for (i = 0; i < threadInfoSize; i++) {
		curl_easy_cleanup(threadInfo[i].curl);
	}

	if (threadInfo != NULL) {
		free(threadInfo);
	}
	
	if (RQ_Close() < 0) {
		RQ_Error = RQ_SCANNER_CLOSE;
		return RQ_Error;
	}

	RQ_Error = RQ_SCANNER_OK;
	return RQ_Error;
}

int RQ_Open(void)
{
	int tableSize, counter, i, offset;

	if (DB_open(&db) < 0) {
		RQ_Error = RQ_SCANNER_OPEN;
		return RQ_Error;
	}

	tableSize = DB_get_lines_num(db);
	counter = ceil((double) tableSize / (double) threadInfoSize);

	offset = 0;
	for (i = 0; i < threadInfoSize; i++) {
		threadInfo[i].db = db;
		threadInfo[i].curl = curl_easy_init();
		if (!threadInfo[i].curl) {
			RQ_Error = RQ_SCANNER_CURL_INIT;
			return RQ_Error;
		}
		
		if (tableSize - counter >= 0) {
			threadInfo[i].nreqs = counter;
			
		} else {
			threadInfo[i].nreqs = tableSize;

		}
		tableSize -= threadInfo[i].nreqs;

		threadInfo[i].offset = offset;
		offset += threadInfo[i].nreqs;
		threadInfo[i].counter = 0;
	}

	RQ_Error = RQ_SCANNER_OK;
	return RQ_Error;
}

int RQ_Close(void)
{
	if (DB_fini() < 0 || DB_close(db) < 0) {
		RQ_Error = RQ_SCANNER_CLOSE;
		return RQ_Error;
	}

	RQ_Error = RQ_SCANNER_OK;
	return RQ_Error;
}

int RQ_SendNextRequest(int threadId, requestInfo_t *curReq)
{
	char *urldb, *urlReq, *method, *descr, *s, *expRes;
	int recId, padd;
	long httpErrorCode;
	CURLcode res;
	scannerRQ_t *thrd;

	if (threadId < 0 || threadId >= threadInfoSize) {
		RQ_Error = RQ_SCANNER_OUT_OF_BOUNDS;
		return RQ_Error;
	}

	thrd = threadInfo + threadId;
	
	/* checks if it has made all requests */
	if (thrd->counter <= thrd->nreqs - 1) {
		recId = DB_go_to_line(thrd->db, thrd->offset + thrd->counter);
		if (recId < 0) {
			RQ_Error = RQ_SCANNER_REQUEST;
			return RQ_Error;
		}
		++thrd->counter;
		urldb = DB_get_field(thrd->db, 1);
		expRes = DB_get_field(thrd->db, 2);
		method = DB_get_field(thrd->db, 3);
		descr = DB_get_field(thrd->db, 4);

		padd = (*urldb == '/') ? 1 : 2;
		if ((urlReq = malloc(strlen(urldb) + strlen(url) + padd)) == NULL) {
			RQ_Error = RQ_SCANNER_NOMEM;
			return RQ_Error;
		}
		
		memcpy(urlReq, url, strlen(url) + 1);
		if (padd == 2) {
			memcpy(urlReq + strlen(urlReq), "/", sizeof(char) + 1);
		}
		memcpy(urlReq + strlen(urlReq), urldb, strlen(urldb) + 1);
	
		/* necessary libcurl calls for sending a request */
		
		/* prepare for sending parameters for POST method */
		if (strncmp(method, "POST", 4) == 0) {
			s = index(urlReq, '?');
			if (s != NULL) {
				*s = '\0';
				curl_easy_setopt(thrd->curl, CURLOPT_POSTFIELDS, ++s);
			}
			
		}

		// create a temporary file for the body of url
		thrd->outputBody = tmpfile();

		curl_easy_setopt(thrd->curl, CURLOPT_URL, urlReq);			/* prepare */
		curl_easy_setopt(thrd->curl, CURLOPT_WRITEDATA, thrd->outputBody);	/* write html body to temp file */
		curl_easy_setopt(thrd->curl, CURLOPT_HEADERFUNCTION, getHeader);	/* parse the headers */
		res = curl_easy_perform(thrd->curl);					/* execute */
		curl_easy_getinfo(thrd->curl, CURLINFO_RESPONSE_CODE, &httpErrorCode);	/* get http status code */

		curReq->description = descr;
		curReq->method = method;
		curReq->httpStatusCode = httpErrorCode;
		curReq->httpExpectedResponse = expRes;
		curReq->httpResponse = statusLine;
		pthread_statusLine_mutex_unlock(&statusLine_mutex);

//		if (md5Calculation == 1 && httpErrorCode == 200) {
		if (httpErrorCode == 200) {
			fseek(thrd->outputBody, 0, SEEK_SET);
			MSG_DIGEST_calcMD5_fromFile(thrd->outputBody, curReq->md5sum);
		}

		// close temporary file
		fclose(thrd->outputBody);

		RQ_Error = RQ_SCANNER_OK;
		return RQ_Error;
		
	} else {
		RQ_Error = RQ_SCANNER_EOR;
		return RQ_Error;
	}
}

int RQ_getURLBody(requestInfo_t *curReq, const char *url)
{
	FILE *outputBody;
	long httpErrorCode;
	CURL *curl;
      
	// create a temporary file for the body of url
	outputBody = tmpfile();
	
	/* necessary libcurl calls for sending a request */
	curl = curl_easy_init();
	curl_easy_setopt(curl, CURLOPT_URL, url);			/* prepare */
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, outputBody);		/* write html body to temp file */
	curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, getHeader);	/* parse the headers */
	curl_easy_perform(curl);					/* execute */
	curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpErrorCode);/* get http status code */

	curReq->description = NULL;
	curReq->method = NULL;
	curReq->httpStatusCode = httpErrorCode;
	curReq->httpExpectedResponse = NULL;
	curReq->httpResponse = NULL;

	fseek(outputBody, 0, SEEK_SET);
	MSG_DIGEST_calcMD5_fromFile(outputBody, curReq->md5sum);

	// close temporary file
	fclose(outputBody);

	RQ_Error = RQ_SCANNER_OK;
	return RQ_Error;

}

size_t getHeader(void *ptr, size_t size, size_t  nmemb,  void  *stream)
{
	int i, ssize;
	char *s = (char *) ptr;
	char *httpResponse = NULL;
	
	if (strncmp(s, "HTTP", 4) == 0) { // parse only the Status Line (HTTP/1.1 ...)
		httpResponse = strchr(s, ' ');

		if (httpResponse == NULL) {
			return -1;
		} else {
			httpResponse++;
			ssize = size * nmemb - (httpResponse - s) - 2; // -2 for stripping the character sequence \r\n
			pthread_statusLine_mutex_lock(&statusLine_mutex);
			if ((statusLine = malloc(ssize + 1)) == NULL) {
				return -1;
			}
			memcpy(statusLine, httpResponse, ssize);
			statusLine[ssize] = '\0';
		}
	}
	return size * nmemb;
}

